/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2016 Frederik Heber. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * StepWorldTimeAction.cpp
 *
 *  Created on: Sep 27, 2016
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"
#include "WorldTime.hpp"

#include <iostream>
#include <string>

#include "Actions/WorldAction/StepWorldTimeAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "StepWorldTimeAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr WorldStepWorldTimeAction::performCall() {
  // create undo state
  int oldtime;
  oldtime = WorldTime::getTime();
  WorldStepWorldTimeState *UndoState = new WorldStepWorldTimeState(oldtime, params);

  if ((oldtime + params.steps.get()) < 0) {
    ELOG(1, "Cannot step back before time step #0.");
    delete UndoState;
    return Action::failure;
  }
  World::getInstance().setTime(oldtime+params.steps.get());
  LOG(0, "Current time step is now: " << WorldTime::getTime() << ".");
  return ActionState::ptr(UndoState);
}

ActionState::ptr WorldStepWorldTimeAction::performUndo(ActionState::ptr _state) {
  WorldStepWorldTimeState *state = assert_cast<WorldStepWorldTimeState*>(_state.get());

  World::getInstance().setTime(state->oldtime);
  LOG(0, "Current time step is now again: " << WorldTime::getTime() << ".");

  return ActionState::ptr(_state);
}

ActionState::ptr WorldStepWorldTimeAction::performRedo(ActionState::ptr _state){
  WorldStepWorldTimeState *state = assert_cast<WorldStepWorldTimeState*>(_state.get());

  World::getInstance().setTime(state->oldtime+state->params.steps.get());
  LOG(0, "Current time step is now: " << WorldTime::getTime() << ".");

  return ActionState::ptr(_state);
}

bool WorldStepWorldTimeAction::canUndo() {
  return true;
}

bool WorldStepWorldTimeAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
